*     JLdL 02Jan21.
*     
*     Copyright (C) 2020-2021 by Jorge L. deLyra <delyra@lmail.if.usp.br>.
*     This program can be copied and/or distributed freely under the
*     GPL license; see the terms and conditions in the files within
*     the sub-directory doc/ of this source directory.
*     
*     A function to return the value of the integral that is shown
*     in Equations (30) and (31) of the paper, given \xi_{1}, \xi_{2}
*     and \xi_{M}; this calculation of the integral does not include
*     the multiplicative factors that appear in front of it in
*     that algebraic equation; the variables passed are:
*     
*     xi1_var_loc : the inner shell radius \xi_{1};
*     xi2_var_loc : the outer shell radius \xi_{2};
*     xim_var_loc : the Schwarzschild radius \xi_{M};
*     xi_del0_loc : the standard variation of \xi;
*     lflg_error  ; an error flag (output).
*     
*     This routine uses the following additional local
*     double-precision temporary variables:
*     
*     polynomial_loc : local temporary for the polynomial;
*     fintegral_loc  : local temporary for the integral;
*     fint_est1_loc  : local temporary for the first function estimate;
*     derv_est1_loc  : local temporary for the first derivative estimate;
*     fint_est2_loc  : local temporary for the second function estimate;
*     derv_est2_loc  : local temporary for the second derivative estimate;
*     fint_aver_loc  : local temporary for the weighted average;
*     fint_delt_loc  : local temporary for the integral increment.
*     
*     This version uses a cubic interpolation of the function in order to
*     calculate the integral, using the values of the function and of its
*     derivative at the two ends of the integration interval.
*     
*     Complex formulas contained in this routine: the integration algorithm.
*     
      FUNCTION FINTEGRAL_FUN
     *     (xi1_var_loc
     *     ,xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_del0_loc
     *     ,lflg_error)
*     
*     Double precision everything; due to the gfortran compiler option
*     -fdefault-real-8, which is in use in the Makefile, this is actually
*     quadruple precision, in the traditional sense of the term, that is,
*     it is actually real*16 (128-bit real data).
      implicit double precision (a-h,o-z)
*     
*     Initialize the error flag.
      lflg_error=0
*     
*     Initialize the RK4 normalization constant.
      rk4_norm=1.0d0/6.0d0
*     
*     Retrieve the value of the polynomial at \xi_{1}.
      polynomial_loc=
     *     polynomial_fun
     *     (xi2_var_loc
     *     ,xim_var_loc
     *     ,xi1_var_loc)
*     
*     Check for the strict positivity of the polynomial; note
*     that this verifies it on the whole integration interval;
*     if this does not succeed, then write out an error message,
*     raise the error flag and return prematurely in error.
      if ( (polynomial_loc.le.0.0d0)
     *     .or.
     *     (xi2_var_loc.le.xim_var_loc) ) then
         write(*,*)
         write(*,*) 'FINTEGRAL_FUN ERROR:'
         write(*,*) '       cubic polynomial not strictly positive'
         write(*,*) '       at \xi_{1} and/or at \xi_{2}, where'
         write(*,*) '       \xi_{1} = ',xi1_var_loc
         write(*,*) '       \xi_{2} = ',xi2_var_loc
         write(*,*) '       \xi_{M} = ',xim_var_loc
         write(*,*) '       Therefore, the integral does not exist'
         write(*,*)
         lflg_error=1
         return
      end if
*     
*     Check for the consistency of the integration limits.
      if (xi2_var_loc.le.xi1_var_loc) then
         write(*,*)
         write(*,*) 'FINTEGRAL_FUN ERROR:'
         write(*,*) '       inconsistent integration limits'
         write(*,*) '       \xi_{1} = ',xi1_var_loc
         write(*,*) '       is not smaller than'
         write(*,*) '       \xi_{2} = ',xi2_var_loc
         write(*,*)
         lflg_error=2
         return
      end if
*     
*     Initialize xi_var, in order to integrate forward.
      xi_var_loc=xi1_var_loc
*     
*     Initialize the right-end integration stop flag.
      lflg_stop=0
*     
*     Initialize the accumulator for the integral.
      fintegral_loc=0.0d0
*     
*     This is using a cubic polynomial interpolation
*     in order to calculate the integral.
*     
*     Initialize the first estimate of the function.
      fint_est1_loc=
     *     fintegrand_fun
     *     (xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_var_loc
     *     ,lflg_error)
*     
*     Initialize the first estimate of the derivative.
      derv_est1_loc=
     *     derivative_fun
     *     (xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_var_loc
     *     ,lflg_error)
*     
*     Calculate the effective variation of \xi.
      xi_del_loc=xi_del0_loc/dsqrt(1.0d0+derv_est1_loc**2)
*     
*     Initialize the second estimate of the function.
      fint_est2_loc=
     *     fintegrand_fun
     *     (xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_var_loc+xi_del_loc
     *     ,lflg_error)
*     
*     Initialize the second estimate of the derivative.
      derv_est2_loc=
     *     derivative_fun
     *     (xi2_var_loc
     *     ,xim_var_loc
     *     ,xi_var_loc+xi_del_loc
     *     ,lflg_error)
*     
*     Loop forward over the integration points.
      do while (xi_var_loc.le.xi2_var_loc)
*     
*     Calculate the weighted averages; note also that the local variable
*     fint_aver_loc is essentially a copy of the first derivative.
         fint_aver_loc=fint_est1_loc+fint_est2_loc
         fint_aver_loc=fint_aver_loc
     *        +rk4_norm*(derv_est1_loc-derv_est2_loc)*xi_del_loc
         fint_aver_loc=0.5d0*fint_aver_loc
         fint_delt_loc=fint_aver_loc*xi_del_loc
*     
*     Increment the variable
         xi_var_loc=xi_var_loc+xi_del_loc
*     
*     Increment the accumulator for the integral.
         fintegral_loc=fintegral_loc+fint_delt_loc
*     
*     Update the first estimate of the function.
         fint_est1_loc=fint_est2_loc
*     
*     Update the first estimate of the derivative.
         derv_est1_loc=derv_est2_loc
*     
*     Update the effective variation of \xi.
         xi_del_loc=xi_del0_loc/dsqrt(1.0d0+derv_est1_loc**2)
*     
*     Stop criterion at the right end; if we overshot the right
*     limit, then set the last point to the value of \xi_{2},
*     and reset the deltas accordingly.
         if ( (xi_var_loc.gt.xi2_var_loc)
     *        .and.
     *        (lflg_stop.eq.0) ) then
            lflg_stop=1
            xi_del_loc=xi_del_loc-(xi_var_loc-xi2_var_loc)
            xi_var_loc=xi2_var_loc
         end if
*     
*     Update the second estimate of the function.
         fint_est2_loc=
     *        fintegrand_fun
     *        (xi2_var_loc
     *        ,xim_var_loc
     *        ,xi_var_loc+xi_del_loc
     *        ,lflg_error)
*     
*     Update the second estimate of the derivative.
         derv_est2_loc=
     *        derivative_fun
     *        (xi2_var_loc
     *        ,xim_var_loc
     *        ,xi_var_loc+xi_del_loc
     *        ,lflg_error)
*     
      end do
*     
*     Pass the function.
      fintegral_fun=fintegral_loc
*     
      return
*     
      END
